from cuda_mock.cuda_mock_impl import *
import sys, os
import time
import socket
import subprocess
import torch

def validation_log_info(*args):
    log(*args, 0)

def validation_log_debug(*args):
    log(*args, 0)

def validation_log_warn(*args):
    log(*args, 1)

class gpu_validation:
    global_op_index = 0

    def __init__(self, model_key, atol, rtol, address, port, cache_dir='/tmp', fallback=False):
        self.model_key = model_key
        self.atol = atol
        self.rtol = rtol

        self.address = address
        self.port = port

        self.cache_dir = cache_dir
        self.fallback = fallback

    def send_tensor(self, tensor):
        self.send_tensor_impl(tensor)
    
    def recv_tensor(self):
        return self.recv_tensor_impl()

    def send_msg(self, msg):
        self.send_msg_impl(msg)
    
    def recv_msg(self):
        return self.recv_msg_impl()

    # lhs is golden tensor
    def tensor_allclose(self, lhs, rhs, lhs_name, rhs_name):
        error_str = ''
        assert lhs_name == rhs_name,f"op_name mismatch {lhs_name} vs {rhs_name}"
        if lhs.shape != rhs.shape:
            error_str = f"shape mismatch {lhs.shape} vs {rhs.shape}"
        elif lhs.dtype != rhs.dtype:
            error_str = f"dtype mismatch {lhs.dtype} vs {rhs.dtype}"
        else:
            error_str = str(torch.allclose(lhs, rhs, self.atol, self.rtol))
        self.result = error_str
        return error_str

    def barrier(self):
        assert False, "not implement"

    def validate(self, tensor):
        assert False, "not implement"

    def get_op_index(self):
        index = gpu_validation.global_op_index
        return index

    def increase_index(self):
        gpu_validation.global_op_index += 1

    def get_tensor_file_name(self):
        return self.model_key + "_" + str(self.port) + "_" + str(self.get_op_index())

    def get_golden_tensor_file_name(self):
        return self.model_key + "_" + str(self.port) + "_" + str(self.get_op_index()) + "_golden"

    def get_tensor_file_path(self):
        return os.path.join(self.cache_dir, self.get_tensor_file_name())


class gpu_validation_master(gpu_validation):

    def validate(self, tensor, op_name):
        self.send_msg(op_name)
        self.send_tensor(tensor)
        self.result = self.recv_result_impl()

        if self.fallback:
            validation_log_debug("recv fallback tensor!")
            self.golden_result = self.recv_tensor()

class gpu_validation_client(gpu_validation):

    def validate(self, tensor, op_name):
        name = self.recv_msg()
        validation_tensor = self.recv_tensor()
        result = self.tensor_allclose(tensor, validation_tensor, op_name, name)
        self.send_result_impl(result)

        if self.fallback:
            validation_log_debug("send fallback tensor!")
            self.send_tensor(tensor)


socket_recv_file_length_buffer_size = 8
socket_recv_buffer_size = 1024 * 1024

def get_file_length(file):
    file.seek(0, 2)
    file_length = file.tell()
    file.seek(0)
    return file_length

def recv_msg(socket):
    byte_data = socket.recv(socket_recv_file_length_buffer_size)
    msg_length = int.from_bytes(byte_data, byteorder='big')
    byte_data = socket.recv(msg_length)
    return byte_data.decode('utf-8')

def send_msg(socket, msg):
    bytedata = msg.encode('utf-8')
    length = len(bytedata)
    blength = length.to_bytes(socket_recv_file_length_buffer_size, byteorder='big')
    socket.sendall(blength)
    socket.sendall(bytedata)

def send_tensor(obj, tensor):
    torch.save(tensor, obj.get_tensor_file_path())
    with open(obj.get_tensor_file_path(), 'rb') as file:
        file_length = get_file_length(file)
        length_bytes = file_length.to_bytes(socket_recv_file_length_buffer_size, byteorder='big')
        obj.connection.sendall(length_bytes)
        while True:
            read_length = file_length if file_length < socket_recv_buffer_size else socket_recv_buffer_size
            binary_data = file.read(read_length)
            validation_log_debug(f"send all data size:{len(binary_data)}")
            obj.connection.sendall(binary_data)
            file_length -= read_length
            if file_length == 0:
                break

def recv_tensor(obj):
    byte_data = obj.connection.recv(socket_recv_file_length_buffer_size)
    file_length = int.from_bytes(byte_data, byteorder='big')
    validation_log_debug(f"validate tensor file size:{file_length}")
    recv_length = 0
    with open(obj.get_tensor_file_path(), 'wb') as file:
        while True:
            byte_data = obj.connection.recv(socket_recv_buffer_size)
            file.write(byte_data)
            recv_length += len(byte_data)
            validation_log_debug(f"current recv data size:{recv_length}")
            if recv_length == file_length:
                break
    return torch.load(obj.get_tensor_file_path())


class socket_gpu_validation_master(gpu_validation_master):
    def __init__(self, model_key, atol, rtol, address, port, cache_dir='/tmp', fallback=False):
        super().__init__(model_key, atol, rtol, address, port, cache_dir, fallback)
        self.get_socket()

    def send_msg_impl(self, msg):
        send_msg(self.connection, msg)

    def recv_msg_impl(self):
        return recv_msg(self.connection)

    def send_tensor_impl(self, tensor):
        send_tensor(self, tensor)

    def recv_tensor_impl(self):
        return recv_tensor(self)

    def recv_result_impl(self):
        return self.recv_msg_impl()

    def get_socket(self):
        server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        server_address = (self.address, self.port)
        validation_log_info('server address {} port {}'.format(*server_address))
        server_socket.bind(server_address)

        server_socket.listen(1)

        validation_log_info('waiting connect...')
        connection, client_address = server_socket.accept()
        validation_log_info(f'get client connect address:{client_address}')
        self.connection = connection
            
    def __del__(self):
        if hasattr(self, "connection"):
            self.connection.close()
        else:
            validation_log_info("has not connect")

class socket_gpu_validation_client(gpu_validation_client):
    def __init__(self, model_key, atol, rtol, address, port, cache_dir='/tmp', fallback=False):
        super().__init__(model_key, atol, rtol, address, port, cache_dir, fallback)
        self.connect()

    def send_msg_impl(self, msg):
        return send_msg(self.connection, msg)

    def recv_msg_impl(self):
        return recv_msg(self.connection)

    def send_tensor_impl(self, tensor):
        send_tensor(self, tensor)
                
    def recv_tensor_impl(self):
        return recv_tensor(self)

    def send_result_impl(self, result):
        return self.send_msg_impl(result)

    def connect(self):
        client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        server_address = (self.address, self.port)
        validation_log_info("connect {} {}".format(*server_address))
        client_socket.connect(server_address)
        self.connection = client_socket

    def __del__(self):
        if hasattr(self, "connection"):
            self.connection.close()
        else:
            validation_log_info("has not connect")

def exec_shell(command):
    validation_log_debug(f"exec command:{command}")
    try:
        result = subprocess.run(command, shell=True, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return result.returncode
    except subprocess.CalledProcessError as e:
        validation_log_warn("Error:", e)
        return -1
    return -1

def sync_pull_file(address, name, cache_dir, fuzzy_matching=False):
    new_file_name = ''
    def find_file():
        nonlocal new_file_name
        for file_name in file_list:
            if fuzzy_matching:
                new_file_name = file_name.split(" ")[-1]
                if new_file_name.startswith(name):
                    return True
            elif file_name.endswith(name):
                new_file_name = name
                return True
        return False

    while True: 
        file_list = os.popen(f"bcecmd bos ls -a {address}").read().strip()
        file_list = file_list.split('\n')
        if find_file():
            break
        time.sleep(0.5)
    bos_file_path = os.path.join(address, new_file_name)
    exec_shell(f"bcecmd bos cp {bos_file_path} {cache_dir}/ -y")
    validation_log_debug(f"pull file:{os.path.join(cache_dir, name)}")

    return os.path.join(cache_dir, new_file_name)

global_bos_msg_index = 0

def encode_path_msg(name, msg):
    global global_bos_msg_index
    result = f"{name}_{global_bos_msg_index}_msg_{msg}"
    global_bos_msg_index += 1
    return result

def bos_send_file(obj, file):
    try_count = 3
    while try_count > 0:
        if exec_shell(f"bcecmd bos cp {file} {obj.address} -y") == 0:
            break
        try_count -= 1
    assert try_count >= 0, "bos_send_file send {file} fail!"

def bos_send_msg(obj, msg):
    msg_path = encode_path_msg(obj.get_tensor_file_path(), msg)
    with open(msg_path, "wt+") as f:
        f.write(msg)
    bos_send_file(obj, msg_path)

def bos_recv_msg(obj):
    global global_bos_msg_index
    op_name_path = sync_pull_file(obj.address, f"{obj.get_tensor_file_name()}_{global_bos_msg_index}_msg_", obj.cache_dir, True)
    global_bos_msg_index += 1
    op_name = ''
    with open(op_name_path, "rt+") as f:
        op_name = f.read().strip()
    bos_name = op_name_path.split('/')[-1]
    assert exec_shell(f"bcecmd bos rm {os.path.join(obj.address, bos_name)} -y")==0, "clear msg fail!"
    return op_name


class bos_gpu_validation_master(gpu_validation_master):
    def __init__(self, model_key, atol, rtol, address, port, cache_dir='/tmp', fallback=False, offline=False):
        super().__init__(model_key, atol, rtol, address, port, cache_dir, fallback)
        if not self.address.endswith('/'):
            self.address += '/'
        self.offline = offline

    def recv_msg_impl(self):
        return bos_recv_msg(self)

    def send_msg_impl(self, msg):
        bos_send_msg(self, msg)

    def send_tensor_impl(self, tensor):
        validation_log_info(self.get_tensor_file_path())
        torch.save(tensor, self.get_tensor_file_path())
        bos_send_file(self, self.get_tensor_file_path())

    def recv_tensor_impl(self):
        tensor_path = sync_pull_file(self.address, self.get_tensor_file_name(), self.cache_dir)
        return torch.load(tensor_path)

    def recv_result_impl(self):
        if self.offline:
            return ""
        return self.recv_msg_impl()

class bos_gpu_validation_client(gpu_validation_client):

    def __init__(self, model_key, atol, rtol, address, port, cache_dir='/tmp', fallback=False, offline=False):
        super().__init__(model_key, atol, rtol, address, port, cache_dir, fallback)
        if not self.address.endswith('/'):
            self.address += '/'
        self.offline = offline

    def send_msg_impl(self, msg):
        bos_send_msg(self, msg)

    def recv_msg_impl(self):
        return bos_recv_msg(self)

    def send_tensor_impl(self, tensor):
        validation_log_info(self.get_tensor_file_path())
        torch.save(tensor, self.get_tensor_file_path())
        bos_send_file(self, self.get_tensor_file_path())

    def recv_tensor_impl(self):
        tensor_path = sync_pull_file(self.address, self.get_tensor_file_name(), self.cache_dir)
        return torch.load(tensor_path)

    def send_result_impl(self, result):
        if self.offline:
            return
        return self.send_msg_impl(result)

token = "test6"
def mytest_bos_master():
    master = bos_gpu_validation_master(token, 1e-3, 1e-3, "", 1000)

    lhs = torch.ones(3, 4)
    rhs = torch.ones(3, 4)
    ret = lhs + rhs
    master.validate(ret)
    master.increase_index()



def mytest_bos_client():
    client = bos_gpu_validation_client(token, 1e-3, 1e-3, "", 1000)

    lhs = torch.ones(3, 4)
    rhs = torch.ones(3, 4)
    ret = lhs + rhs
    client.validate(ret)
    client.increase_index()

SOCKET_PORT = 1000
TEST_SHAPE = (1024, 2, 4)

def mytest_socket_master():
    validation = socket_gpu_validation_master(token, 1e-3, 1e-3, "127.0.0.1", SOCKET_PORT)

    lhs = torch.ones(*TEST_SHAPE)
    rhs = torch.ones(*TEST_SHAPE)
    ret = lhs + rhs
    validation.validate(ret)
    validation.increase_index()

    ret = lhs - rhs
    validation.validate(ret)
    validation.increase_index()

def mytest_socket_client():
    validation = socket_gpu_validation_client(token, 1e-3, 1e-3, "127.0.0.1", SOCKET_PORT, "./")

    lhs = torch.ones(*TEST_SHAPE)
    rhs = torch.ones(*TEST_SHAPE)
    ret = lhs + rhs
    validation.validate(ret)
    validation.increase_index()

    ret = lhs - rhs
    validation.validate(ret)
    validation.increase_index()

import torch
from torch.utils._pytree import tree_flatten, tree_map
import functools
from torch.utils._python_dispatch import TorchDispatchMode

class GpuValidation(TorchDispatchMode):
    '''
    master is the device will be validation
    client is golden device 
    '''
    def __init__(self, is_gpu, model_key, atol, rtol, address, port=SOCKET_PORT, cache_dir='/tmp', use_bos=False, fallback=False, offline=False):
        self.is_gpu = is_gpu
        self.fallback = fallback
        self.offline = offline
        if not use_bos:
            master = socket_gpu_validation_master
            client = socket_gpu_validation_client
        else:
            master = bos_gpu_validation_master
            client = bos_gpu_validation_client
        if is_gpu:
            self.validation = client(model_key, atol, rtol, address, port, cache_dir, fallback, offline=offline)
            validation_log_debug(f"initialize client validation completed!")
        else:
            self.validation = master(model_key, atol, rtol, address, port, cache_dir, fallback, offline=offline)
            validation_log_debug(f"initialize master validation completed!")

    def __torch_dispatch__(self, op, types, dev_args=(), dev_kwargs=None):
        validation_log_info(f"will call op:{op}")

        result = op(*dev_args, **dev_kwargs)
        
        if isinstance(result, torch.Tensor):
            self.validation.validate(result, f"{op}")
            if not self.offline or self.is_gpu:
                validation_log_info(f"validate op:{op} dtype:{result.dtype} shape:{result.shape} result:{self.validation.result}")
            if self.fallback and not self.is_gpu:
                self.validation.increase_index()
                return self.validation.golden_result
            self.validation.increase_index()
        else:
            validation_log_info(f'skip not tensor result:{type(result)} op:{op}')

        return result


import torch
import torch.nn as nn

class SimpleNN(nn.Module):
    def __init__(self, input_size, hidden_size, output_size):
        super(SimpleNN, self).__init__()
        self.fc1 = nn.Linear(input_size, hidden_size)  # 输入层到隐藏层的全连接层
        self.relu = nn.ReLU()  # 非线性激活函数
        self.fc2 = nn.Linear(hidden_size, output_size)  # 隐藏层到输出层的全连接层

    def forward(self, x):
        x = self.fc1(x)  # 输入层到隐藏层的线性变换
        x = self.relu(x)  # 非线性激活函数
        x = self.fc2(x)  # 隐藏层到输出层的线性变换
        return x

def mytest_validation_master():
    with GpuValidation(False, token, 1e-3, 1e-3, "127.0.0.1", SOCKET_PORT, './'):
        input_size = 5
        hidden_size = 10
        output_size = 2
        model = SimpleNN(input_size, hidden_size, output_size)

        # 准备输入数据
        input_data = torch.randn(3, input_size)  # 创建一个3x5的随机输入张量

        # 将输入数据传递给模型，获取模型的输出
        output = model(input_data)

        # 打印模型输出
        print("模型输出：", output)

def mytest_validation_client():
    with GpuValidation(True, token, 1e-3, 1e-3, "127.0.0.1", SOCKET_PORT):
        input_size = 5
        hidden_size = 10
        output_size = 2
        model = SimpleNN(input_size, hidden_size, output_size)

        # 准备输入数据
        input_data = torch.randn(3, input_size)  # 创建一个3x5的随机输入张量

        # 将输入数据传递给模型，获取模型的输出
        output = model(input_data)

        # 打印模型输出
        print("模型输出：", output)

if __name__ == "__main__":
    if sys.argv[1] == "1":
        validation_log_info("client")
        mytest_validation_client()
    else:
        validation_log_info("master")
        mytest_validation_master()



import os
import ctypes
from .dynamic_obj import *
import json
import re
import inspect
from .ctypes_helper import convert_arg_list_of_str

script_dir = os.path.dirname(os.path.abspath(__file__))
cuda_mock_impl = ctypes.CDLL(f'{script_dir}/libcuda_mock_impl.so')
cuda_mock_impl.print_hook_initialize.argtypes = [ctypes.POINTER(ctypes.c_char_p), ctypes.POINTER(ctypes.c_char_p)]

def add(lhs, rhs):
    return lhs + rhs

def initialize():
    return cuda_mock_impl.initialize()

def uninitialize():
    return cuda_mock_impl.uninitialize()

def internal_install_hook(*args):
    new_args = [ctypes.c_char_p(arg.encode('utf-8')) for arg in args]
    return cuda_mock_impl.internal_install_hook(*new_args)

def internal_install_hook_regex(*args):
    lib = ctypes.CDLL(args[-2])
    new_args = [ctypes.c_char_p(arg.encode('utf-8')) for arg in args]
    cuda_mock_impl.internal_install_hook_regex(*new_args)
    return lib

def xpu_initialize():
    return cuda_mock_impl.xpu_initialize()

def print_hook_initialize(target_libs, target_symbols):
    target_libs = convert_arg_list_of_str(target_libs)
    target_symbols = convert_arg_list_of_str(target_symbols)
    cuda_mock_impl.print_hook_initialize(target_libs, target_symbols)
    

def patch_runtime():
    return cuda_mock_impl.patch_runtime()

def log(*args, level=0):
    caller_frame = inspect.currentframe().f_back
    caller_filename = inspect.getframeinfo(caller_frame).filename
    caller_lineno = inspect.getframeinfo(caller_frame).lineno
    new_args = [f'[{caller_filename}:{caller_lineno}]{arg}' for arg in args]
    new_args = [ctypes.c_char_p(arg.encode('utf-8')) for arg in new_args]
    if level == 0:
        return cuda_mock_impl.py_log_info(*new_args)
    elif level == 1:
        return cuda_mock_impl.py_log_warn(*new_args)
    else:
        return cuda_mock_impl.py_log_error(*new_args)

is_nvidia_gpu = os.environ.get('CUDA_VERSION', None) or os.environ.get('NVIDIA_VISIBLE_DEVICES', None)

class ProfileDataCollection:
    def __init__(self, device):
        self.data = []
        self.device = device
    def append_gpu(self, key, data):
        self.load_from_cache()
        self.append_implement(key, data, 'gpu')
        self.dump_to_cache()

    def append_xpu(self, key, data):
        self.load_from_cache()
        self.append_implement(key, data, 'xpu')
        self.dump_to_cache()

    def append_implement(self, key, data, device):
        data = self.statistic_data(data, device)
        for it in self.data:
            if key in it:
                it[key][device] = data
                return
        self.data.append({key : {device:data}})

    def get_xpu_time(self, data):
        return float(data[-1].strip().replace("ns", ""))

    def get_gpu_time(self, data):
        if data[0].startswith("aten::empty"):
            return 0.0
        if data[0] != 'total':
            return 0.0
        time_str = data[1].strip()
        if time_str.endswith("ns"):
            return float(time_str.replace("ns", ""))
        elif time_str.endswith("us"):
            return float(time_str.replace("us", "")) * 1000
        elif time_str.endswith("ms"):
            return float(time_str.replace("ms", "")) * 1000 * 1000
        elif time_str.endswith("s"):
            return float(time_str.replace("s", "")) * 1000 * 1000 * 1000
        raise RuntimeError(f"error uint:{time_str}")

    def statistic_data(self, data, device):
        total = 0.0
        for it in data:
            if is_nvidia_gpu:
                total += self.get_gpu_time(it)
            else:
                total += self.get_xpu_time(it)
        data = {"total" : total, "data" : data}
        return data

    def load_from_cache(self):
        pass
        # try:
        #     with open(f"{self.device}-profile_data.json", "rt") as f:
        #         self.data = json.load(f)
        # except Exception:
        #     pass

    def dump_to_cache(self):
        with open(f"{self.device}-profile_data.json", "wt") as f:
            json.dump(self.data, f, indent=4)

gProfileDataCollection = ProfileDataCollection("gpu" if is_nvidia_gpu else "xpu")
gDefaultTargetLib = ["libxpucuda.so", "libcuda.so"]
gDefaultTargetSymbols = ["__printf_chk", "printf","fprintf","__fprintf","vfprintf",]
class __XpuRuntimeProfiler:
    def __init__(self, target_libs = gDefaultTargetLib, target_symbols = gDefaultTargetSymbols):
        print_hook_initialize(target_libs=target_libs, target_symbols=target_symbols)
    def start_capture(self):
        cuda_mock_impl.print_hook_start_capture()

    def end_capture(self, op_key):
        log(f"op_key:{op_key}")
        c_python_object = ctypes.py_object(self)
        cuda_mock_impl.print_hook_end_capture(c_python_object)
        log(f"data:{self.data}")
        gProfileDataCollection.append_xpu(op_key, self.data)
        return self.data

    def end_callback(self, data):
        pattern = r"\[\s*?XPURT_PROF\s*?\]\s+(\S+?)\s+(\d+?)\s+(\S+?)\s+ns"
        matches = re.findall(pattern, data)
        self.data = matches


class __GpuRuntimeProfiler:
    def __init__(self, target_libs = gDefaultTargetLib, target_symbols = gDefaultTargetSymbols):
        print_hook_initialize(target_libs=target_libs, target_symbols=target_symbols)
    def start_capture(self):
        import torch
        from torch.profiler import profile, record_function, ProfilerActivity
        self.prof = profile(activities=[ProfilerActivity.CUDA], record_shapes=False).__enter__()
    def end_capture(self, op_key):
        self.prof.__exit__(None, None, None)
        # print(self.prof.table(row_limit=-1))
        table = self.prof.key_averages().table(sort_by="cuda_time_total", row_limit=-1)
        time_list = self.prof.key_averages()

        data = []
        for it in time_list:
            data.append((it.key, it.cuda_time_total_str))
            #print(it)
            #print(it.key)
            #print(it.self_cuda_time_total_str)
            #print(it.cuda_time_str)
        table = str(table)
        log(f"{table}")

        pattern = r"Self CUDA time total:(.*\s)"
        matches = re.findall(pattern, table)

        if len(matches) == 1:
            data.append(('total', matches[0]))
        else:
            data.append(('total', "0.0ms"))
        self.data = data
        gProfileDataCollection.append_gpu(op_key, self.data)
        log(f"{op_key}:{self.data}")

RuntimeProfiler = __GpuRuntimeProfiler if is_nvidia_gpu else __XpuRuntimeProfiler


class HookInstaller:
    def __init__(self, lib):
        self.lib = ctypes.CDLL(lib)
        c_python_object = ctypes.py_object(self)
        c_string_lib = ctypes.c_char_p(lib.encode('utf-8'))
        cuda_mock_impl.create_hook_installer(c_python_object, c_string_lib)
    def is_target_lib(self, name):
        print(f"HookInstaller is_taget_lib:{name}")
        assert False, "unimplement"

    def is_target_symbol(self, name):
        print(f"HookInstaller is_target_symbol:{name}")
        assert False, "unimplement"

    def new_symbol_name(self, name):
        return name


str_code = '''
#include <stddef.h>
#include <stdio.h>
#include <unistd.h>

#include <fstream>
#include <iostream>

#include "logger/logger.h"
#include "profile/Timer.h"

#define EXPORT __attribute__((__visibility__("default")))

extern "C" {{

void* __origin_{0} = nullptr;
void* __origin_{1} = nullptr;
void* __origin_{2} = nullptr;


EXPORT int {0}(const void* arg, uint64_t size, uint64_t offset) {{
    hook::Timer timer(__func__);
    int ret = reinterpret_cast<decltype(&{0})>(__origin_{0})(arg, size, offset);
    return ret;
}}

EXPORT int {1}(void* func) {{
    hook::Timer timer(__func__);
    int ret = reinterpret_cast<decltype(&{1})>(__origin_{1})(func);
    return ret;
}}

EXPORT int {2}(int nclusters, int ncores, void* stream) {{
    hook::Timer timer(__func__);
    int ret = reinterpret_cast<decltype(&{2})>(__origin_{2})(nclusters, ncores, stream);
    return ret;
}}

}}
'''

def xpu_runtime_profile():
    script_dir = os.path.dirname(os.path.abspath(__file__))
    funcs = ["xpu_launch_argument_set", "xpu_launch_async", "xpu_launch_config"]
    lib = dynamic_obj(str_code.format(*funcs), True).appen_compile_opts('-lpthread').compile().get_lib()
    internal_install_hook_regex(r"libxpurt\.so\..*", r"[^(libxpurt\.so\..*)]", "xpu_launch_argument_set", str(lib), "xpu_launch_argument_set")
    internal_install_hook_regex(r"libxpurt\.so\..*", r"[^(libxpurt\.so\..*)]", "xpu_launch_async", str(lib), "xpu_launch_async")
    internal_install_hook_regex(r"libxpurt\.so\..*", r"[^(libxpurt\.so\..*)]", "xpu_launch_config", str(lib), "xpu_launch_config")





from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext
import sys
import setuptools
import os
import subprocess
from setuptools.extension import Extension as _Extension
from glob import glob
import shutil

script_dir = os.path.dirname(os.path.abspath(__file__))

publish_build = False
if os.environ.get('PUBLISH_BUILD'):
    publish_build = True

class CMakeExtension(_Extension):
    def __init__(self, name, sourcedir=''):
        super().__init__(name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)

class CMakeBuild(build_ext):
    def run(self):
        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):

        ninja_args = []
        install_cmd = 'make'
        if not publish_build:
            ninja_args = ['-GNinja']
            install_cmd = 'ninja'

        cmake_args = [
            '-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name))),
            '-DPYTHON_EXECUTABLE=' + sys.executable,
            '-DCMAKE_INSTALL_PREFIX=' + os.path.join(os.path.abspath(os.path.dirname(self.build_lib)), os.path.basename(self.build_lib)),
            '-DENABLE_BUILD_WITH_GTEST=OFF',
            # f'-B {os.path.join(script_dir, "build")}',
        ]
        if publish_build:
            cmake_args.append('-DPYTHON_INCLUDE_DIR=' + os.environ.get('PYTHON_INCLUDE_DIR'))
            cmake_args.append('-DPYTHON_LIBRARY=' + os.environ.get('PYTHON_LIBRARY'))
            cmake_args.append('-DPUBLISH_BUILD=ON')

        build_dir = os.path.abspath(os.path.join(self.build_temp, ext.name))
        os.makedirs(build_dir, exist_ok=True)

        subprocess.check_call(['cmake', f'{script_dir}'] + cmake_args + ninja_args, cwd=build_dir)
        subprocess.check_call(['cmake', '--build', '.'], cwd=build_dir)
        subprocess.check_call([f'{install_cmd}', 'install'], cwd=build_dir)

setup(
    name="cuda-mock",
    version="0.1.6",
    author="lipracer",
    author_email="lipracer@gmail.com",
    description="a tools hook some api call at runtime",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',

    url="https://github.com/lipracer/torch-cuda-mock",

    ext_modules=[CMakeExtension('cuda_mock_impl', sourcedir='.')],
    cmdclass=dict(build_ext=CMakeBuild),
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    package_data={'' : ['*']},
    zip_safe=False,
)



import ctypes

def convert_arg_list_of_str(list_of_str):
    N = len(list_of_str)
    arr = (ctypes.c_char_p * (N + 1))()
    
    arr[:-1] = [s.encode('utf-8') for s in list_of_str]
    arr[-1] = None
    return arr